from abc import ABCMeta, abstractmethod


class Index(metaclass=ABCMeta):
    """
    索引
    """

    @abstractmethod
    def exists(self, id: str) -> bool:
        """
        指定ID的数据是否存在

        :param id: 数据ID
        """

    @abstractmethod
    def get(self, id: str) -> dict:
        """
        返回指定数据

        :param id: 数据的ID
        """

    @abstractmethod
    def put(self, id: str, data: dict):
        """
        更新指定数据，指定ID不存在时自动创建数据

        :param id: 数据ID
        :param data: 数据内容
        """

    @abstractmethod
    def delete(self, id: str):
        """
        删除指定数据

        :param id: 数据ID
        """


class Search(metaclass=ABCMeta):
    """
    提供全文检索服务
    """

    @abstractmethod
    def index(self, topic: str) -> Index:
        """
        对指定topic返回索引

        :param topic: topic名称
        """



from kikyo import Kikyo

from kikyo.bundle.datahub import PulsarBasedDataHub
from kikyo.bundle.oss import MinioBasedOSS, AliyunOSS
from kikyo.bundle.search import EsBasedSearch


def configure_kikyo(client: Kikyo):
    PulsarBasedDataHub(client)
    MinioBasedOSS(client)
    AliyunOSS(client)
    EsBasedSearch(client)



from elasticsearch import Elasticsearch

from kikyo import Kikyo
from kikyo.search import Search, Index


class EsBasedSearch(Search):
    def __init__(self, client: Kikyo):
        settings = client.settings.deep('elasticsearch')

        if not settings:
            return

        kwargs = {
            'hosts': settings['hosts']
        }
        if 'username' in settings:
            kwargs['http_auth'] = (settings['username'], settings['password'])
        if 'timeout' in settings:
            kwargs['timeout'] = settings['timeout']

        self.es = Elasticsearch(**kwargs)
        self.index_prefix = settings.get('index_prefix', default='')

        client.add_component('es_search', self)

    def get_index_of_topic(self, topic):
        if topic.startswith(self.index_prefix):
            return topic
        return f'{self.index_prefix}{topic}'

    def index(self, topic: str) -> Index:
        return EsIndex(topic, self)


class EsIndex(Index):
    def __init__(self, topic: str, client: EsBasedSearch):
        self.topic = topic
        self.es = client.es
        self.client = client

    def exists(self, id: str) -> bool:
        return self.es.exists(
            self.client.get_index_of_topic(self.topic),
            id,
        )

    def get(self, id: str) -> dict:
        resp = self.es.get(
            self.client.get_index_of_topic(self.topic),
            id,
        )
        return resp['_source']

    def put(self, id: str, data: dict):
        self.es.index(
            self.client.get_index_of_topic(self.topic),
            body=data,
            id=id,
        )

    def delete(self, id: str):
        self.es.delete(
            self.client.get_index_of_topic(self.topic),
            id,
        )



import re
import sys
from os.path import join, dirname

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

with open(join(dirname(__file__), 'README.rst'), 'r', encoding='utf-8') as fd:
    long_description = fd.read()


def read_version():
    p = join(dirname(__file__), 'kikyo', '__init__.py')
    with open(p, 'r', encoding='utf-8') as f:
        return re.search(r"__version__ = '([^']+)'", f.read()).group(1)


def read_requirements(file):
    with open(join(dirname(__file__), 'requirements', file), 'r', encoding='utf-8') as f:
        return [l.strip() for l in f]


class PyTest(TestCommand):
    def run_tests(self):
        import pytest

        errno = pytest.main(['tests'])
        sys.exit(errno)


tests_require = read_requirements('test.txt')
install_requires = [
    'PyYAML>=5.4.1',
    'requests>=2.25.1',
    'pulsar-client>=2.7.0,<2.8',
    'minio>=7.0.1,<8.0',
    'elasticsearch>=7.13.1,<7.14',
    'oss2>=2.14.0',
    'fastavro>=0.24.0',
]


def main():
    if sys.version_info < (3, 6):
        raise RuntimeError('The minimal supported Python version is 3.6')

    setup(
        name='kikyo',
        version=read_version(),
        description='kikyo package',
        long_description=long_description,
        author='jadbin',
        author_email='jadbin.com@hotmail.com',
        license='MIT',
        zip_safe=False,
        packages=find_packages(exclude=('tests',)),
        include_package_data=True,
        python_requires='>=3.6',
        install_requires=install_requires,
        tests_require=tests_require,
        cmdclass={'test': PyTest},
        entry_points={'kikyo.plugins': 'kikyo_bundle = kikyo.bundle.config'},
        classifiers=[
            'License :: OSI Approved :: MIT License',
            'Intended Audience :: Developers',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Topic :: Software Development :: Libraries :: Python Modules',
        ],
    )


if __name__ == '__main__':
    main()
